//
// Copyright (c) 1996 Vermeer Technologies, Inc., a wholly owned
//               subsidiary of Microsoft Corp.  All Rights Reserved

// CCount implementation
#include "cntfile.h"

#ifdef WIN32
#include <windows.h>
#include <io.h>
#include <share.h>
#endif

#ifdef UNIX
#include <signal.h>
#include <unistd.h>
#include <sys/types.h>
#endif

#include <errno.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

CCount::CCount(const char *filename, BOOL create)
{

    // Assume the worst
    m_count = 0;
    error = TRUE;

    // Open the file and lock it.  Timeout for waiting for lock is 10 sec
    if (!OpenAndLock(filename, 10, create))    
        return;

    m_count = ReadCount();
    error = FALSE;
}

CCount::~CCount()
{
    WriteCount(m_count);
    Close();            // Closes and unlocks the file    
}

long CCount::GetCount()
{
    return m_count;
}

void CCount::SetCount(long newcount)
{
    m_count = newcount;
}

void CCount::Increment(long amount)
{
    m_count += amount;
}


#ifdef UNIX

#ifdef SGI
void alarm_handler(...)
#else
void alarm_handler(int /*sig*/)
#endif 
{
    // catch alarm signal and ignore
}
#endif


BOOL CCount::OpenAndLock(const char *filename, int nTimeoutSec, BOOL create)
{

#ifdef WIN32
    DWORD startTime = GetTickCount();
    DWORD nTimeout = nTimeoutSec * 1000;        // Timeout in milleseconds
    for (fd = -1; nTimeoutSec == 0 || GetTickCount() - startTime < nTimeout;)
    {
        // Try to open the file in locked mode 
        fd = _sopen(filename, _O_RDWR, _SH_DENYWR);

        // Success, exit the for loop
        if (fd != -1)
            break;

        // No success. If file was not found, and create is specified, then
        // create it, lock it, and write a 0 to it
        if (errno == ENOENT && create)
        {
            fd = _sopen(filename, _O_RDWR | _O_CREAT, _SH_DENYWR, _S_IREAD | _S_IWRITE);
            WriteCount(0);
            break;
        }

        // Otherwise, delay half a second, and try again.
        Sleep(500);
    }
    if (fd == -1)
        return FALSE;        // Timed out

#endif /* WIN32 */


#ifdef UNIX
    fd = open(filename, O_RDWR);
    if (fd == -1)
    {
        if (errno == ENOENT && create)
        {
            // If it's okay to create it, try to do so
            fd = open(filename, O_RDWR | O_CREAT, 0666);
            if (fd == -1)        // still failed, abort
                return FALSE;
            WriteCount(0);        // Fill in with count of 0
        }
        else
            return FALSE;
    }

    // Setup the alarm handler
    #ifdef SGI
    void (*func)(...) = 0;
    #else
    void (*func)(int) = 0;
    #endif
    func = signal(SIGALRM, alarm_handler);
    if ((void *)(func) == (void *)(SIG_ERR))
    {
        close(fd);
        fd = -1;
        return FALSE;
    }
    
    // Set number of seconds for alarm:
    alarm(nTimeoutSec);

    // Setup the lock to be the whole file
    struct flock sflock;
    sflock.l_type = F_WRLCK;
    sflock.l_whence = 0;
    sflock.l_start = 0;
    sflock.l_len = 0;

    // Try to do the lock.  This blocks until lock is achieved
    // or the alarm goes off.
    int ret = fcntl(fd, F_SETLKW, &sflock);

    // If ret=-1, it failed
    if (ret == -1)
    {
        close(fd);
        fd = -1;
        return FALSE;
    }
#endif /* UNIX */

    // Check validity of the count file
    if (!IsValid())
    {
        fd = -1;
        return FALSE;
    }
    return TRUE;
}





void CCount::Close()
{
    close(fd);
}


long CCount::ReadCount()
{
    lseek(fd, KEY_LEN, SEEK_SET);
    char buffer[MAX_NUM];
    int nBytesRead = read(fd, buffer, MAX_NUM);
    if (nBytesRead == -1)
    {
        error = TRUE;
        return 0;
    }
    buffer[nBytesRead] = '\0';        // Close off the string
    return atol(buffer);            // convert to number
}


void CCount::WriteCount(long count)
{    
    lseek(fd, 0, SEEK_SET);
    write (fd, KEY_STR, KEY_LEN);

    char buffer[MAX_NUM];
    sprintf(buffer, "%011ld", count);
    int nStrLen = strlen(buffer);
    write(fd, buffer, nStrLen);
}



BOOL CCount::IsValid()
{
    lseek(fd, 0, SEEK_SET);
    char buffer[KEY_LEN];
    int nBytesRead = read(fd, buffer, KEY_LEN);
    if (nBytesRead == -1)
    {
        error = TRUE;
        return FALSE;
    }
    return (strncmp(buffer, KEY_STR, KEY_LEN) == 0);
}

            
