//
// Copyright (c) 1996 Vermeer Technologies, Inc., a wholly owned
//               subsidiary of Microsoft Corp.  All Rights Reserved

// hitcount.cpp : Defines the initialization routines for the DLL.
//

#include "fpcount.h"
#include "cntfile.h"
#include "../../webbot.h"
#include <string.h>
#include <stdlib.h>

#ifdef UNIX
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#else
#include <direct.h>
#include <io.h>
#endif



////////////////////////////////////////////////////////////////////////
// Utility URL and filename manipulation functions.


// Extracts just the filename portion of the URL.
// that is, a URL of "/simple/foo/default.htm" becomes "default.htm",
// and "blah.html" would become "blah.html"
const char* URLtoFileName(const char *URL)
{
    if (!URL)
        return "";

    // Look for the last '/' in the URL:
    const char *last = URL;
    for (const char *p=URL; *p; p++)
        if (*p == '/')
            last = p+1;
    return last;
}




// Copies the PageURL directory name (the PageURL minus the page itself) into 
// the destination.  It also translates the '/' character into the character
// specified by translateChar.  This is useful if the desired final form
// is to contain '\' or ':' as the directory separator.  If the PageURL
// is NULL, it does nothing.
// Returns the number of characters copied, not counting the '\0' at the end
// NOTE: this routine assumes enough space has been allocated in the 
// destination buffer to hold the *entire* PageURL, not just enough up
// to the '/' of the PageURL.
//
// Example: PageURL="Matt/simple/index.htm" and translateChar is '\' then
// the result in dest is "Matt\simple"

void CopyPageDirName(const char *PageURL, char *dest, char translateChar)
{
    if (!PageURL)
        return;
    const char *pSource = PageURL;    
    char *pDest = dest;
    char *last = pDest;
    for (; *pSource; pSource++, pDest++)
    {
        if (*pSource == '/')
        {
            last = pDest;
            *pDest = translateChar;
        }
        else
            *pDest = *pSource;
    }

    // last points to the last '/' in the PageURL, or the beginning of it if
    // there were no '/' in the string.  Thus, close off the string at
    // this location.

    // if characters were copied then append a dir seperator to the end
    // of the whole thing.  Otherwise, just close off the string    
    if (last != dest)
    {
        *last = translateChar;
        *(last+1) = '\0';
    }
    else    
        *last = '\0';            
}


const char *SubWebName(CWebBotDict &bot)
{
    // Returns the subweb name, taken from the WebURL.
    // Assumes WebURL is in the form
    // protocol://host/subweb
    //
    // Method to find the subweb: Scan for the first / which
    // does not have a / following it.  
    const char *WebURL = bot.GetValue("WebURL");
    if (!WebURL)
        return "";
    
    const char *p;
    for (p = WebURL; *p; p++)
    {
        if (*p=='/')
        {
            if (*(p+1) != '/')
                return p;
            else
                p++;            // Skip over next spot
        }
    }
    return p;

}




///////////////////////////////////////////////////////////////////
// Counter file functions

void CreateAndResetCounterFile(const char *counterFile, long resetValue)
{
    // Create a CCount object for dealing with the count file.
    // Waits until the file is available, then locks it.  Times out
    // if the file does not become available in a certain amount of time.
    // If the file does not exist, it will create it.
    CCount countfile(counterFile, TRUE);
    countfile.SetCount(resetValue);

    // CCount destructor writes and unlocks the file
}


void HandleCounterFile(CWebBotDict &bot)
{    
    // Figures out the filename for the counter.  
    // and makes sure it exists with a correct count according to the
    // reset key.  If the reset key does not exist, and the file doesn't
    // exist, make a count beginning at 0.

    // The counter file name will be the name of the page with .cnt
    // appended to the end, stored in the _private directory


    const char *PageURL = bot.GetValue("PageURL");
    const char *PageFileName = URLtoFileName(PageURL);
    const char *DocumentRoot = bot.GetValue("DocumentRoot");    

    if (!PageURL || !DocumentRoot)
        return;

    // The idea is that the CountFile is a filename which is relative to
    // the location of the .htm file which contains the counter tag.
    // In order to "mess around with" the counter file, we need to know
    // the absolute pathname to the file.  The DocumentRoot tells us
    // where the root directory of the web is (e.g. c:\inetsrv\wwwroot\Matt), 
    // and the PageURL tells us what is the web-relative URL of the 
    // .htm file which called us (e.g. simple/index.htm).  
    // So, strip off the filename of the PageURL, and append that to
    // the end of the DocumentRoot, changing all '/' to the DIR_SEPARATOR
    // for this platform.

    // Create a buffer large enough:
    int DocumentRootLen = strlen(DocumentRoot);
    char *PageDirName = new char[DocumentRootLen + 1 +        // +1 for DIR_SEPARATOR
                            strlen(PageURL) + 1 +        // +1 for DIR_SEPARATOR
                            + 9    +                        // +9 for "_private/"
                            strlen(PageFileName) + 5];    // +5 for ".cnt"+'\0'
        
 
    // Put in the document root:
    strcpy(PageDirName, DocumentRoot);

    // Append the DIR_SEPARATOR
    PageDirName[DocumentRootLen] = DIR_SEPARATOR;

    // Append the PageURL, changing all '/' to the local machine's DIR_SEPARATOR
    // Also appends the DIR_SEPERATOR to the end of the string
    CopyPageDirName(PageURL, &PageDirName[DocumentRootLen+1], DIR_SEPARATOR);

    strcat(PageDirName, "_private/");

    // Check for existence of, and create directory
    if (access(PageDirName, 0) != 0)
    {
#ifdef UNIX
        mkdir(PageDirName, 0777);
#else
        mkdir(PageDirName);
#endif
    }
    
    // Now append the Page filename
    strcat(PageDirName, PageFileName);

    // Now append the ".cnt" ending
    strcat(PageDirName, ".cnt");

    ////////////////////////////////////////////////////
    // Now buffer holds the filename of the count file.

    // If we are to reset it, do so:
    const char *reset;
    const char *resetValue;
    reset = bot.GetValue("B-Reset");

    if (    (reset) && 
        (*reset=='T') && 
        (resetValue = bot.GetValue("I-ResetValue"))
       )

    {        
        // Only if reset key is TRUE, and there's a reset value
        CreateAndResetCounterFile(PageDirName, atol(resetValue));
        bot.SetValue("B-Reset", "FALSE");
    }
    else
    {
        // If the file doesn't exist then create it with a count of 0
        if (access(PageDirName, 0) != 0)
            CreateAndResetCounterFile(PageDirName, 0);
    }

    delete[] PageDirName;
}


const int QueryNumTranslations = 2;
const char QueryTranslateFrom[QueryNumTranslations] = {' ', '\\'};
const char *QueryTranslateTo[QueryNumTranslations] = {"+", "/"};

const int URLNumTranslations = 2;
const char URLTranslateFrom[URLNumTranslations] = {' ', '\\'};
const char *URLTranslateTo[URLNumTranslations] = {"%20", "/"};


char *URLEncode(const char *url, 
                const char *translateFrom, 
                const char **translateTo, 
                int numTranslations)
{
    // Note: Users of this function should delete the pointer themselves
    if (url == NULL)
        return NULL;

    // maximum size of the encoded string is the url's len*3.  This assumes that
    // no TranslateTo string is more than 3 chars, which will always be
    // true for URL encoding
    char *encoded = new char[strlen(url)*3 + 1];            // +1 for '\0'

    char *currentTo = encoded;
    BOOL found;
    for (const char *currentFrom = url; *currentFrom; currentFrom++)
    {
        found = FALSE;
        for (int i = 0; i < numTranslations && !found; i++)
        {
            if (*currentFrom == translateFrom[i])
            {
                // Matches a char, copy over the translateTo for this char
                strcpy(currentTo, translateTo[i]);
                currentTo+=strlen(translateTo[i]);
                found = TRUE;
            }
        }

        if (!found)
        {
            // Wasn't a char to translate, just do plain vanilla copy
            *currentTo = *currentFrom;
            currentTo++;
        }
    }
    // Close it off
    *currentTo = '\0';    
    return encoded;
}


/////////////////////////////////////////////////////////////////////
// DLL Entry point for expanding the WebBot


// cgi and form are unused, so they are commented out to prevent compiler
// warnings.
BeginWebBotExpand(HitCounter,ret,bot,cgi,form)
{
    // Squelch compiler warnings
    (void)cgi;
    (void)form;

    // Take care of the counter file
    HandleCounterFile(bot);

    // Create the HTML tag which will call the cgi program "fpcount"

    // First, generate the PageDirName
    const char *PageURL = bot.GetValue("PageURL");
    if (!PageURL)
        PageURL = "";

    // The +2 is for the potentially extra '/' at the end, and for the '\0'
    char *PageDirName = new char[strlen(PageURL) + 2];
    CopyPageDirName(PageURL, PageDirName, '/');

    // Now generate the Page Filename
    const char *PageFileName = URLtoFileName(PageURL);


    // create the URL.  

    // Method:
    // Create the URL without the query string in ret, URL encode it into "url"
    // Create the query string in ret, do a query string encoding into "query"
    // Clear ret, put in the MIME headers, append url and query

    // First generate URL:    
    ret.Clear();

    ret.Append("/cgi-bin/fpcount.exe");
    ret.Append(SubWebName(bot));
    ret.Append("/");    
    ret.Append(PageDirName);
    ret.Append("?");
    char *url = URLEncode(ret.GetContents(), 
                            URLTranslateFrom, 
                            URLTranslateTo, 
                            URLNumTranslations);        

    // Next generate the query string:
    ret.Clear();
    ret.Append("Page=");
    ret.Append(PageFileName);
    
    const char *szTemp;
    char* custom = 0;
    if ((szTemp = bot.GetValue("U-Custom")) && szTemp[0] != '\0')
    {
        ret.Append("|Custom=");
        ret.Append(szTemp);
        custom = URLEncode(szTemp,
                           URLTranslateFrom, 
                           URLTranslateTo, 
                           URLNumTranslations);        
    }
    else if ((szTemp = bot.GetValue("I-Image")))
    {
        // Only if no Custom image...
        ret.Append("|Image=");
        ret.Append(szTemp);
    }
    
    if ((szTemp = bot.GetValue("I-Digits")) && szTemp[0] != '0' )
    {
        // Only if non-zero fixed number of digits
        ret.Append("|Digits=");
        ret.Append(szTemp);
    }        

    // Now query encode the query string:
    char *query = URLEncode(ret.GetContents(), 
                            QueryTranslateFrom, 
                            QueryTranslateTo, 
                            QueryNumTranslations);    

    // Put in the MIME Links: header so that the explorer knows
    // that our WebBot is linking to the count file.  That is, append
    // Links: _private/PageFilename.cnt.  But URL encode the filename
    ret.Clear();
    ret.Append("Links: ");
    ret.Append("_private/");
    char *EncodedPageFileName = URLEncode(PageFileName,
                                          URLTranslateFrom,
                                          URLTranslateTo,
                                          URLNumTranslations);
    ret.Append(EncodedPageFileName);
    ret.Append(".cnt");
    if (custom)
    {
        ret.Append(" ");
        ret.Append(custom);
    }
    ret.Append("\n\n");

    ret.Append("<img src=\"");
    ret.Append(url);
    ret.Append(query);
    ret.Append("\">");

    
    delete[] PageDirName;
    delete[] EncodedPageFileName;
    delete[] url;
    delete[] query;
    delete[] custom;
}
EndWebBotExpand



