//
// Copyright (c) 1996 Vermeer Technologies, Inc., a wholly owned
//               subsidiary of Microsoft Corp.  All Rights Reserved

#include <fstream.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#include <stdio.h>

#ifdef WIN32
#include <io.h>
#include <strstrea.h>
#define IOS_BINARY ios::binary
#endif

#ifdef UNIX
#include <strstream.h>
#define IOS_BINARY ios::bin
#endif

#include "cntfile.h"
#include "numimage.h"
#include "fpcount.h"

#include "image0.h"
#include "image1.h"
#include "image2.h"
#include "image3.h"
#include "image4.h"


// Parse the query string.  It looks for the keys "Page", "Image",
// and "Custom".  It does a case-sensitive search. 
// WARNING: do not use the params string after calling ParseQueryString
// because the parse method mangles the params string in the process.
// this is done for efficiency to avoid copying strings.
void ParseQueryString(char *params,
                      char **countFileName,
                      int *imageNum,
                      char **customImageFile,
                      int *nFixedDigits)
{    
    // Default value for the image number if we don't find an image tag
    *imageNum = -1;

    char *pKeyBegin = params;
    char *pKeyEnd = NULL;    
    for (char *p = params; *p;)
    {
        if (*p == '=')        
            pKeyEnd = p;
        // If it's a key divider, or the last char of the string,
        // parse the entire key/value pair.  But if we're at the end
        // of the string, move p there so that the rest of the code
        // works as it is expecting.
        if (*(p+1) == '\0')
            p++;
        if (*p == '|' || *p == '\0')
        {            
            // At this point, key is from pKeyBegin to pKeyEnd-1,
            // and value is from pKeyEnd+1 to p-1.
            
            if (pKeyEnd)
            {
                int keyLen = pKeyEnd - pKeyBegin;    
                if (keyLen >0)
                {
                    // Replace the | seperator with a '\0' so we can just break
                    // up this string to return the countFileName and the
                    // customImageFile strings.  That way, we don't need to
                    // do a strcpy.
                    *p = '\0';
                    if (strncmp("Page", pKeyBegin, keyLen) == 0)            
                        *countFileName = pKeyEnd+1;            
                    else if(strncmp("Image", pKeyBegin, keyLen) == 0)                
                        *imageNum = atoi(pKeyEnd+1);            
                    else if (strncmp("Custom", pKeyBegin, keyLen) == 0)
                        *customImageFile = pKeyEnd+1;
                    else if(strncmp("Digits", pKeyBegin, keyLen) == 0)                
                        *nFixedDigits = atoi(pKeyEnd+1);            

                }
            }
            // Reset the pointers:
            pKeyBegin = p+1;
            pKeyEnd = NULL;
        }
        p++;    
    }

}    


const int QueryNumDecodes = 2;
const char QueryFromDecode[QueryNumDecodes] = {'+', '/'};
const char QueryToDecode[QueryNumDecodes] = {' ', DIR_SEPARATOR};

// Performs in-place decoding of the query string
char *QueryDecode(char *queryString)
{
    // Simple char by char decoding.
    if (queryString == NULL)
        return NULL;
    for (char *p = queryString; *p; p++)
    {    
        for (int i=0; i<QueryNumDecodes; i++)
        {
            if (*p == QueryFromDecode[i])
            {
                *p = QueryToDecode[i];
                break;
            }
        }
    }
    return queryString;
}


// argc and argv are unused, so they are commented out to prevent compiler
// warnings.
int main(
    int /* argc */,
    char ** /* argv */)
{
    long count;
    char *params = QueryDecode(getenv("QUERY_STRING"));
    char *path = getenv("PATH_TRANSLATED");
    if (path == NULL)
        path = "";

    char *pageFileName = NULL;
    int imageNum;
    char *customImageFile = "";
    int nFixedDigits = -1;
    
    if (params == NULL)
        return 1;
    
    // Parse the query string.  Fills imageNum with -1 if it's a
    // custom image
    ParseQueryString(params, &pageFileName, &imageNum, &customImageFile, &nFixedDigits);    

    // If we didn't get a Page filename, just give up.
    if (pageFileName == NULL || strlen(pageFileName) == 0)
        return 1;
    
    // Create full countfile filename and custom image filename by
    // concatenating the path and the relative filenames.
    char *fullCountFileName = new char[strlen(path) + 
                                        9 +               // +9 for "_private/"
                                        strlen(pageFileName) +
                                        5];               // +4 for ".cnt"+'\0'
    sprintf(fullCountFileName, "%s_private%c%s.cnt", 
            path, DIR_SEPARATOR, pageFileName);
    
    char *fullCustomFileName = new char[strlen(path) + 
                                        strlen(customImageFile) + 1];
    strcpy(fullCustomFileName, path);
    strcat(fullCustomFileName, customImageFile);
    
    // The constructor waits until the file is no longer locked, then locks it.
    CCount *countFile = new CCount(fullCountFileName);        
    countFile->Increment();    
    count = countFile->GetCount();
    // Destruct as soon as possible to unlock the file so others can access it.
    delete countFile;

    // For now, just suppose we are doing a memory file
    CNumberImage image;

    istream *file = NULL;    
    if (imageNum != -1)
    {
        switch (imageNum)
        {
        default:
        case 0:
            file = new istrstream((char *)image0, IMAGE_0_NUM_BYTES); break;
        case 1:
            file = new istrstream((char *)image1, IMAGE_1_NUM_BYTES); break;
        case 2:
            file = new istrstream((char *)image2, IMAGE_2_NUM_BYTES); break;
        case 3:
            file = new istrstream((char *)image3, IMAGE_3_NUM_BYTES); break;
        case 4:
            file = new istrstream((char *)image4, IMAGE_4_NUM_BYTES); break;
        }
    }
    else
    {
        file = new ifstream(fullCustomFileName, ios::in | IOS_BINARY | ios::nocreate);        
    }

    if (file == NULL)
        cout << "Error reading image template file "
             << fullCustomFileName << endl;

    if (!image.CreateImage(*file, count, (nFixedDigits != -1), nFixedDigits))
        cout << "Error creating image file" << endl;

#ifdef WIN32
    _setmode( _fileno( stdout ), _O_BINARY );    // Make stdout binary
#endif
#ifdef UNIX
    fdopen( fileno(stdout), "wb");               // Make stdout binary
#endif

    fprintf(stdout, "Content-Type: image/gif\n\n");
    image.OutputImageAsGif(stdout);
    delete file;
    delete fullCountFileName;
    delete fullCustomFileName;

    fflush(stdout);
    return 0;
}
